/*
 * IRC - Internet Relay Chat, ircd/socks.c
 * Copyright (C) 1999 Christopher L. Bond (face)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "sys.h"
#include <sys/socket.h>
#include "h.h"
#include "struct.h"
#include "common.h"
#include "socks.h"
#include "s_bsd.h"
#include "ircd.h"

RCSTAG_CC("$Id: socks.c,v 1.11 1999/06/26 14:45:58 face Exp $");

#ifdef CHECK_SOCKS

void start_socks(aClient *cptr)
{
  struct sockaddr_in sin;

  if ((cptr->socksfd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
  {
    Debug((DEBUG_ERROR, "Cannot create SOCKS socket for %p", cptr));
    return;
  }

  if (cptr->socksfd >= (MAXCONNECTIONS - 2))
  {
    close(cptr->socksfd);
    cptr->socksfd = -1;
    return;
  }

  set_non_blocking(cptr->socksfd, cptr);

  sin.sin_family = AF_INET;
  sin.sin_port = htons(SOCKS_PORT);

  memcpy((char *)&sin.sin_addr.s_addr, (char *)&cptr->ip,
      sizeof(unsigned long));

#ifdef CONNECT_INFO
  send(cptr->fd, "NOTICE AUTH :*** Checking for open SOCKS server\n", 48, 0);
#endif

  if ((connect(cptr->socksfd, (struct sockaddr *)&sin, sizeof(sin)) < 0) &&
      (errno != EINPROGRESS))
  {
#ifdef CONNECT_INFO
    send(cptr->fd, "NOTICE AUTH :*** No SOCKS server found\n", 39, 0);
#endif

    close(cptr->socksfd);
    cptr->socksfd = -1;
    return;
  }

  cptr->socks_time = time(0);
  cptr->exflags |= (FLAGS_SOCKS | FLAGS_WRSOCKS);
  if (cptr->socksfd > highest_fd)
    highest_fd = cptr->socksfd;
}

void send_socks(aClient *cptr)
{
  unsigned char buffer[9];
  unsigned long addr;

  if (cptr->socksfd < 0)
    return;

  addr = htonl(cptr->ip.s_addr);

  buffer[0] = 4;
  buffer[1] = 1;
  buffer[2] = (((unsigned short)SOCKS_PORT) >> 8);
  buffer[3] = (((unsigned short)SOCKS_PORT) & 0xFF);
  buffer[4] = (addr >> 24);
  buffer[5] = ((addr >> 16) & 0xFF);
  buffer[6] = ((addr >> 8) & 0xFF);
  buffer[7] = (addr & 0xFF);
  buffer[8] = buffer[9] = 0;

  if (send(cptr->socksfd, buffer, 9, 0) != 9)
    no_socks(cptr);

  cptr->exflags &= ~FLAGS_WRSOCKS;
}

void read_socks(aClient *cptr)
{
  int length;
  unsigned char buffer[9];

  if (cptr->socksfd < 0)
  {
    if ((!DoingAuth(cptr)) && (!DoingDNS(cptr)))
      SetAccess(cptr);

    return;
  }

  length = recv(cptr->socksfd, buffer, 9, 0);
  close(cptr->socksfd);

  cptr->exflags &= ~FLAGS_SOCKS;

  if (cptr->socksfd == highest_fd)
    while (!loc_clients[highest_fd])
      highest_fd--;

  cptr->socksfd = -1;

  if (length < 4)
  {
#ifdef CONNECT_INFO
    send(cptr->fd, "NOTICE AUTH :*** No SOCKS server found\n", 39, 0);
#endif

    if ((!DoingAuth(cptr)) && (!DoingDNS(cptr)))
      SetAccess(cptr);

    return;
  }

  if (buffer[1] == 90)
  {
#ifdef CONNECT_INFO
    send(cptr->fd, "NOTICE AUTH :*** Unsecured SOCKS server found\n", 46, 0);
#endif

    exit_client(cptr, cptr, &me, "Open SOCKS server.  Refer to "
	"http://www.galaxynet.org/proxy for more information.");
    return;
  }
}

void no_socks(aClient *cptr)
{
#ifdef CONNECT_INFO
  send(cptr->fd, "NOTICE AUTH :*** No SOCKS server found\n", 39, 0);
#endif

  if ((!DoingAuth(cptr)) && (!DoingDNS(cptr)))
    SetAccess(cptr);

  cptr->exflags &= ~FLAGS_SOCKS;
  close(cptr->socksfd);
  cptr->socksfd = -1;

  if (cptr->socksfd == highest_fd)
    while (!loc_clients[highest_fd])
      highest_fd--;
}
#endif
